@@ -37,7 +37,6 @@ try:
     import syslog
 except ImportError:
     syslog = None
-import traceback
 
 from oslo_config import cfg
 from oslo_utils import importutils
@@ -191,9 +190,7 @@ class KeywordArgumentAdapter(BaseLoggerAdapter):
 def _create_logging_excepthook(product_name):
     def logging_excepthook(exc_type, value, tb):
         extra = {'exc_info': (exc_type, value, tb)}
-        getLogger(product_name).critical(
-            "".join(traceback.format_exception_only(exc_type, value)),
-            **extra)
+        getLogger(product_name).critical('Unhandled error', **extra)
     return logging_excepthook
 
 
@@ -232,10 +229,8 @@ def _load_log_config(log_config_append):
 def _mutate_hook(conf, fresh):
     """Reconfigures oslo.log according to the mutated options."""
 
-    # verbose is deprecated, so I didn't make it mutable, so there's no need to
-    # test for it here.
     if (None, 'debug') in fresh:
-        _refresh_root_level(conf.debug, conf.verbose)
+        _refresh_root_level(conf.debug)
 
     if (None, 'log-config-append') in fresh:
         _load_log_config.old_time = 0
@@ -327,25 +322,17 @@ def _find_facility(facility):
     return getattr(syslog, facility)
 
 
-def _refresh_root_level(debug, verbose):
+def _refresh_root_level(debug):
     """Set the level of the root logger.
 
-    If 'debug' is True, the level will be DEBUG. Otherwise we look at 'verbose'
-    - if that is True, the level will be INFO. If neither are set, the level
-    will be WARNING.
-
-    Note the 'verbose' option is deprecated.
-
-    :param debug
-    :param verbose
+    :param debug: If 'debug' is True, the level will be DEBUG.
+     Otherwise the level will be INFO.
     """
     log_root = getLogger(None).logger
     if debug:
         log_root.setLevel(logging.DEBUG)
-    elif verbose:
-        log_root.setLevel(logging.INFO)
     else:
-        log_root.setLevel(logging.WARNING)
+        log_root.setLevel(logging.INFO)
 
 
 def _setup_logging_from_conf(conf, project, version):
@@ -370,10 +357,15 @@ def _setup_logging_from_conf(conf, project, version):
         streamlog = handlers.ColorHandler()
         log_root.addHandler(streamlog)
 
-    elif not logpath:
+    if conf.use_journal:
+        journal = handlers.OSJournalHandler()
+        log_root.addHandler(journal)
+
+    # if None of the above are True, then fall back to standard out
+    if not logpath and not conf.use_stderr and not conf.use_journal:
         # pass sys.stdout as a positional argument
         # python2.6 calls the argument strm, in 2.7 it's stream
-        streamlog = logging.StreamHandler(sys.stdout)
+        streamlog = handlers.ColorHandler(sys.stdout)
         log_root.addHandler(streamlog)
 
     if conf.publish_errors:
@@ -396,7 +388,7 @@ def _setup_logging_from_conf(conf, project, version):
                                                          version=version,
                                                          datefmt=datefmt,
                                                          config=conf))
-    _refresh_root_level(conf.debug, conf.verbose)
+    _refresh_root_level(conf.debug)
 
     for pair in conf.default_log_levels:
         mod, _sep, level_name = pair.partition('=')
@@ -424,6 +416,11 @@ def _setup_logging_from_conf(conf, project, version):
 _loggers = {}
 
 
+def get_loggers():
+    """Return a copy of the oslo loggers dictionary."""
+    return _loggers.copy()
+
+
 def getLogger(name=None, project='unknown', version='unknown'):
     """Build a logger with the given name.
 
@@ -459,3 +456,8 @@ def get_default_log_levels():
     setup.
     """
     return list(_options.DEFAULT_LOG_LEVELS)
+
+
+def is_debug_enabled(conf):
+    """Determine if debug logging mode is enabled."""
+    return conf.debug
