@@ -74,6 +74,20 @@ def _fake_context():
     return ctxt
 
 
+def _fake_new_context():
+    # New style contexts have a user_name / project_name, this is done
+    # distinctly from the above context to not have to rewrite all the
+    # other tests.
+    ctxt = context.RequestContext(1, 1, overwrite=True)
+    ctxt.user_name = 'myuser'
+    ctxt.project_name = 'mytenant'
+    ctxt.domain = 'mydomain'
+    ctxt.project_domain = 'myprojectdomain'
+    ctxt.user_domain = 'myuserdomain'
+
+    return ctxt
+
+
 class CommonLoggerTestsMixIn(object):
     """These tests are shared between LoggerTestCase and
     LazyLoggerTestCase.
@@ -107,13 +121,6 @@ class CommonLoggerTestsMixIn(object):
         self.log.info("foo", context=_fake_context())
         self.assertTrue(True)  # didn't raise exception
 
-    def test_will_be_verbose_if_verbose_flag_set(self):
-        self.config(verbose=True)
-        logger_name = 'test_is_verbose'
-        log.setup(self.CONF, logger_name)
-        logger = logging.getLogger(logger_name)
-        self.assertEqual(logging.INFO, logger.getEffectiveLevel())
-
     def test_will_be_debug_if_debug_flag_set(self):
         self.config(debug=True)
         logger_name = 'test_is_debug'
@@ -121,12 +128,12 @@ class CommonLoggerTestsMixIn(object):
         logger = logging.getLogger(logger_name)
         self.assertEqual(logging.DEBUG, logger.getEffectiveLevel())
 
-    def test_will_not_be_verbose_if_verbose_flag_not_set(self):
-        self.config(verbose=False)
-        logger_name = 'test_is_not_verbose'
+    def test_will_be_info_if_debug_flag_not_set(self):
+        self.config(debug=False)
+        logger_name = 'test_is_not_debug'
         log.setup(self.CONF, logger_name)
         logger = logging.getLogger(logger_name)
-        self.assertEqual(logging.WARN, logger.getEffectiveLevel())
+        self.assertEqual(logging.INFO, logger.getEffectiveLevel())
 
     def test_no_logging_via_module(self):
         for func in ('critical', 'error', 'exception', 'warning', 'warn',
@@ -297,6 +304,63 @@ class OSSysLogHandlerTestCase(BaseTestCase):
             syslog.syslog.assert_called_once_with(syslog.LOG_INFO, msg_unicode)
 
 
+class OSJournalHandlerTestCase(BaseTestCase):
+    """Test systemd journal logging.
+
+    This is a lightweight test for testing systemd journal logging. It
+    mocks out the journal interface itself, which allows us to not
+    have to have systemd-python installed (which is not possible to
+    install on non Linux environments).
+
+    Real world testing is also encouraged.
+
+    """
+    def setUp(self):
+        super(OSJournalHandlerTestCase, self).setUp()
+        self.config(use_journal=True)
+        self.journal = mock.patch("oslo_log.handlers.journal").start()
+        self.addCleanup(self.journal.stop)
+        log.setup(self.CONF, 'testing')
+
+    def test_emit(self):
+        l = log.getLogger('nova-test.foo')
+        local_context = _fake_new_context()
+        l.info("Foo", context=local_context)
+        self.assertEqual(
+            self.journal.send.call_args,
+            mock.call(mock.ANY, CODE_FILE=mock.ANY, CODE_FUNC='test_emit',
+                      CODE_LINE=mock.ANY, LOGGER_LEVEL='INFO',
+                      LOGGER_NAME='nova-test.foo', PRIORITY=6,
+                      SYSLOG_IDENTIFIER=mock.ANY,
+                      REQUEST_ID=mock.ANY,
+                      PROJECT_NAME='mytenant',
+                      PROCESS_NAME='MainProcess',
+                      THREAD_NAME='MainThread',
+                      USER_NAME='myuser'))
+
+    def test_emit_exception(self):
+        l = log.getLogger('nova-exception.foo')
+        local_context = _fake_new_context()
+        try:
+            raise Exception("Some exception")
+        except Exception:
+            l.exception("Foo", context=local_context)
+        self.assertEqual(
+            self.journal.send.call_args,
+            mock.call(mock.ANY, CODE_FILE=mock.ANY,
+                      CODE_FUNC='test_emit_exception',
+                      CODE_LINE=mock.ANY, LOGGER_LEVEL='ERROR',
+                      LOGGER_NAME='nova-exception.foo', PRIORITY=3,
+                      SYSLOG_IDENTIFIER=mock.ANY,
+                      REQUEST_ID=mock.ANY,
+                      EXCEPTION_INFO=mock.ANY,
+                      EXCEPTION_TEXT=mock.ANY,
+                      PROJECT_NAME='mytenant',
+                      PROCESS_NAME='MainProcess',
+                      THREAD_NAME='MainThread',
+                      USER_NAME='myuser'))
+
+
 class LogLevelTestCase(BaseTestCase):
     def setUp(self):
         super(LogLevelTestCase, self).setUp()
@@ -309,8 +373,7 @@ class LogLevelTestCase(BaseTestCase):
         levels.append(warn_level + '=WARN')
         levels.append(other_level + '=7')
         levels.append(trace_level + '=TRACE')
-        self.config(default_log_levels=levels,
-                    verbose=True)
+        self.config(default_log_levels=levels)
         log.setup(self.CONF, 'testing')
         self.log = log.getLogger(info_level)
         self.log_no_debug = log.getLogger(warn_level)
@@ -338,6 +401,11 @@ class LogLevelTestCase(BaseTestCase):
         l = log.getLogger('nova-trace.foo')
         self.assertEqual(log.TRACE, l.logger.getEffectiveLevel())
 
+    def test_get_loggers(self):
+        log._loggers['sentinel_log'] = mock.sentinel.sentinel_log
+        res = log.get_loggers()
+        self.assertDictEqual(log._loggers, res)
+
 
 class JSONFormatterTestCase(LogTestBase):
     def setUp(self):
@@ -434,6 +502,26 @@ class JSONFormatterTestCase(LogTestBase):
         self.log.info(b'%s', u'\u2622'.encode('utf8'))
         self.assertIn(expected, self.stream.getvalue())
 
+    def test_exception(self):
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        try:
+            raise RuntimeError('test_exception')
+        except RuntimeError:
+            self.log.warning('testing', context=ctxt)
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('error_summary', data)
+        self.assertEqual('RuntimeError: test_exception',
+                         data['error_summary'])
+
+    def test_no_exception(self):
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        self.log.info('testing', context=ctxt)
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('error_summary', data)
+        self.assertEqual('', data['error_summary'])
+
 
 def get_fake_datetime(retval):
     class FakeDateTime(datetime.datetime):
@@ -473,6 +561,7 @@ class FluentFormatterTestCase(LogTestBase):
         self.log.debug(test_msg, test_data, key='value', context=local_context)
 
         data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('lineno', data)
         self.assertIn('extra', data)
         extra = data['extra']
         self.assertEqual('value', extra['key'])
@@ -488,6 +577,24 @@ class FluentFormatterTestCase(LogTestBase):
         self.assertEqual('DEBUG', data['level'])
         self.assertFalse(data['traceback'])
 
+    def test_exception(self):
+        local_context = _fake_context()
+        try:
+            raise RuntimeError('test_exception')
+        except RuntimeError:
+            self.log.warning('testing', context=local_context)
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('error_summary', data)
+        self.assertEqual('RuntimeError: test_exception',
+                         data['error_summary'])
+
+    def test_no_exception(self):
+        local_context = _fake_context()
+        self.log.info('testing', context=local_context)
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('error_summary', data)
+        self.assertEqual('', data['error_summary'])
+
     def test_json_exception(self):
         test_msg = 'This is %s'
         test_data = 'exceptional'
@@ -580,6 +687,64 @@ class ContextFormatterTestCase(LogTestBase):
                                                six.text_type(message))
         self.assertEqual(expected, self.stream.getvalue())
 
+    def test_exception_logging(self):
+        # NOTE(dhellmann): If there is an exception and %(error_summary)s
+        # does not appear in the format string, ensure that it is
+        # appended to the end of the log lines.
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        try:
+            raise RuntimeError('test_exception_logging')
+        except RuntimeError:
+            self.log.warning(message, context=ctxt)
+        expected = 'RuntimeError: test_exception_logging\n'
+        self.assertTrue(self.stream.getvalue().endswith(expected))
+
+    def test_skip_logging_builtin_exceptions(self):
+        # NOTE(dhellmann): Several of the built-in exception types
+        # should not be automatically added to the log output.
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        ignored_exceptions = [
+            ValueError, TypeError, KeyError, AttributeError, ImportError
+        ]
+        for ignore in ignored_exceptions:
+            try:
+                raise ignore('test_exception_logging')
+            except ignore as e:
+                self.log.warning(message, context=ctxt)
+                expected = '{}: {}'.format(e.__class__.__name__, e)
+            self.assertNotIn(expected, self.stream.getvalue())
+
+    def test_exception_logging_format_string(self):
+        # NOTE(dhellmann): If the format string includes
+        # %(error_summary)s then ensure the exception message ends up in
+        # that position in the output.
+        self.config(logging_context_format_string="A %(error_summary)s B")
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        try:
+            raise RuntimeError('test_exception_logging')
+        except RuntimeError:
+            self.log.warning(message, context=ctxt)
+        expected = 'A RuntimeError: test_exception_logging'
+        self.assertTrue(self.stream.getvalue().startswith(expected))
+
+    def test_no_exception_logging_format_string(self):
+        # NOTE(dhellmann): If there is no exception but the format
+        # string includes %(error_summary)s then ensure the "-" is
+        # inserted.
+        self.config(logging_context_format_string="%(error_summary)s")
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        self.log.info(message, context=ctxt)
+        expected = '-\n'
+        self.assertTrue(self.stream.getvalue().startswith(expected))
+
     def test_unicode_conversion_in_adapter(self):
         ctxt = _fake_context()
         ctxt.request_id = six.text_type('99')
@@ -674,6 +839,12 @@ class ContextFormatterTestCase(LogTestBase):
         self.log.info(b'%s', u'\u2622'.encode('utf8'))
         self.assertIn(expected, self.stream.getvalue())
 
+    def test_dict_args_with_unicode(self):
+        msg = '%(thing)s'
+        arg = {'thing': '\xc6\x91\xc6\xa1\xc6\xa1'}
+        self.log.info(msg, arg)
+        self.assertIn(arg['thing'], self.stream.getvalue())
+
 
 class ExceptionLoggingTestCase(LogTestBase):
     """Test that Exceptions are logged."""
@@ -690,7 +861,7 @@ class ExceptionLoggingTestCase(LogTestBase):
         except Exception:
             excepthook(*sys.exc_info())
 
-        expected_string = ("CRITICAL somename [-] "
+        expected_string = ("CRITICAL somename [-] Unhandled error: "
                            "Exception: Some error happened")
         self.assertIn(expected_string, self.stream.getvalue(),
                       message="Exception is not logged")
@@ -765,15 +936,16 @@ class FancyRecordTestCase(LogTestBase):
         warncolor = handlers.ColorHandler.LEVEL_COLORS[logging.WARN]
         info_msg = 'info'
         warn_msg = 'warn'
-        infoexpected = "%s %s %s\n" % (infocolor, keyed_log_string, info_msg)
-        warnexpected = "%s %s %s\n" % (warncolor, keyed_log_string, warn_msg)
+        infoexpected = "%s %s %s" % (infocolor, keyed_log_string, info_msg)
+        warnexpected = "%s %s %s" % (warncolor, keyed_log_string, warn_msg)
 
         self.colorlog.info(info_msg, context=ctxt)
-        self.assertEqual(infoexpected, self.stream.getvalue())
+        self.assertIn(infoexpected, self.stream.getvalue())
         self.assertEqual('\033[00;36m', infocolor)
 
         self.colorlog.warn(warn_msg, context=ctxt)
-        self.assertEqual(infoexpected + warnexpected, self.stream.getvalue())
+        self.assertIn(infoexpected, self.stream.getvalue())
+        self.assertIn(warnexpected, self.stream.getvalue())
         self.assertEqual('\033[01;33m', warncolor)
 
     def test_fancy_key_in_log_msg(self):
@@ -831,51 +1003,47 @@ class InstanceRecordTestCase(LogTestBase):
         fake_resource = {'uuid': uuid}
         message = 'info'
         self.log.info(message, context=ctxt, instance=fake_resource)
-        expected = ('[%s]: [instance: %s] %s\n' %
-                    (ctxt.request_id, uuid, message))
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % uuid
+        self.assertIn(expected, self.stream.getvalue())
 
     def test_instance_dict_in_default_log_msg(self):
         uuid = 'C9B7CCC6-8A12-4C53-A736-D7A1C36A62F3'
         fake_resource = {'uuid': uuid}
         message = 'info'
         self.log.info(message, instance=fake_resource)
-        expected = '[instance: %s] %s\n' % (uuid, message)
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % uuid
+        self.assertIn(expected, self.stream.getvalue())
 
     def test_instance_uuid_as_arg_in_context_log_msg(self):
         ctxt = _fake_context()
         uuid = 'C9B7CCC6-8A12-4C53-A736-D7A1C36A62F3'
         message = 'info'
         self.log.info(message, context=ctxt, instance_uuid=uuid)
-        expected = ('[%s]: [instance: %s] %s\n' %
-                    (ctxt.request_id, uuid, message))
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % uuid
+        self.assertIn(expected, self.stream.getvalue())
 
     def test_instance_uuid_as_arg_in_default_log_msg(self):
         uuid = 'C9B7CCC6-8A12-4C53-A736-D7A1C36A62F3'
         message = 'info'
         self.log.info(message, instance_uuid=uuid)
-        expected = '[instance: %s] %s\n' % (uuid, message)
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % uuid
+        self.assertIn(expected, self.stream.getvalue())
 
     def test_instance_uuid_from_context_in_context_log_msg(self):
         ctxt = _fake_context()
         ctxt.instance_uuid = 'CCCCCCCC-8A12-4C53-A736-D7A1C36A62F3'
         message = 'info'
         self.log.info(message, context=ctxt)
-        expected = ('[%s]: [instance: %s] %s\n' %
-                    (ctxt.request_id, ctxt.instance_uuid, message))
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % ctxt.instance_uuid
+        self.assertIn(expected, self.stream.getvalue())
 
     def test_resource_uuid_from_context_in_context_log_msg(self):
         ctxt = _fake_context()
         ctxt.resource_uuid = 'RRRRRRRR-8A12-4C53-A736-D7A1C36A62F3'
         message = 'info'
         self.log.info(message, context=ctxt)
-        expected = ('[%s]: [instance: %s] %s\n' %
-                    (ctxt.request_id, ctxt.resource_uuid, message))
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % ctxt.resource_uuid
+        self.assertIn(expected, self.stream.getvalue())
 
     def test_instance_from_context_in_context_log_msg(self):
         # NOTE: instance when passed in a context object is just a uuid.
@@ -884,9 +1052,8 @@ class InstanceRecordTestCase(LogTestBase):
         ctxt.instance = 'IIIIIIII-8A12-4C53-A736-D7A1C36A62F3'
         message = 'info'
         self.log.info(message, context=ctxt)
-        values = (ctxt.request_id, ctxt.instance, message)
-        expected = '[%s]: [instance: %s] %s\n' % values
-        self.assertEqual(expected, self.stream.getvalue())
+        expected = '[instance: %s]' % ctxt.instance
+        self.assertIn(expected, self.stream.getvalue())
 
 
 class TraceLevelTestCase(LogTestBase):
@@ -1067,14 +1234,12 @@ class MutateTestCase(BaseTestCase):
         log_root = log.getLogger(None).logger
         log._setup_logging_from_conf(self.CONF, 'test', 'test')
         self.assertEqual(False, self.CONF.debug)
-        self.assertEqual(True, self.CONF.verbose)
         self.assertEqual(log.INFO, log_root.getEffectiveLevel())
 
         shutil.copy(paths[1], paths[0])
         self.CONF.mutate_config_files()
 
         self.assertEqual(True, self.CONF.debug)
-        self.assertEqual(True, self.CONF.verbose)
         self.assertEqual(log.DEBUG, log_root.getEffectiveLevel())
 
     @mock.patch.object(logging.config, "fileConfig")
@@ -1313,14 +1478,12 @@ class LogConfigOptsTestCase(BaseTestCase):
         f = six.StringIO()
         self.CONF([])
         self.CONF.print_help(file=f)
-        for option in ['debug', 'verbose', 'log-config', 'watch-log-file']:
+        for option in ['debug', 'log-config', 'watch-log-file']:
             self.assertIn(option, f.getvalue())
 
-    def test_debug_verbose(self):
-        self.CONF(['--debug', '--verbose'])
-
+    def test_debug(self):
+        self.CONF(['--debug'])
         self.assertEqual(True, self.CONF.debug)
-        self.assertEqual(True, self.CONF.verbose)
 
     def test_logging_opts(self):
         self.CONF([])
@@ -1591,3 +1754,23 @@ class LoggerNameTestCase(LoggerTestCase):
         expected = logger_name.replace('_', '.')
         l = log.getLogger(logger_name)
         self.assertEqual(expected, l.logger.name)
+
+
+class IsDebugEnabledTestCase(test_base.BaseTestCase):
+    def setUp(self):
+        super(IsDebugEnabledTestCase, self).setUp()
+        self.config_fixture = self.useFixture(
+            fixture_config.Config(cfg.ConfigOpts()))
+        self.config = self.config_fixture.config
+        self.CONF = self.config_fixture.conf
+        log.register_options(self.config_fixture.conf)
+
+    def _test_is_debug_enabled(self, debug=False):
+        self.config(debug=debug)
+        self.assertEqual(debug, log.is_debug_enabled(self.CONF))
+
+    def test_is_debug_enabled_off(self):
+        self._test_is_debug_enabled()
+
+    def test_is_debug_enabled_on(self):
+        self._test_is_debug_enabled(debug=True)
